"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var atob = require("atob");
var lodash_1 = require("lodash");
var Cursor_1 = require("../../types/Cursor");
var PaginationDirection_1 = require("../../types/PaginationDirection");
var SortOrder_1 = require("../../types/SortOrder");
var xor = function (conditionA, conditionB) {
    return (conditionA && !conditionB) || (!conditionA && conditionB);
};
var getCursorKeyFilter = function (sortKey, sort, pagination, cursorObj) {
    var sortOrder = lodash_1.get(sort, sortKey);
    var ascendingPagination = !xor(sortOrder === SortOrder_1.asc, pagination.direction === PaginationDirection_1.forward);
    var cursorValue = lodash_1.get(cursorObj, sortKey);
    if (ascendingPagination) {
        if (sortKey === 'id') {
            return { $gt: cursorValue };
        }
        else {
            return { $gte: cursorValue };
        }
    }
    else {
        if (sortKey === 'id') {
            return { $lt: cursorValue };
        }
        else {
            return { $lte: cursorValue };
        }
    }
};
exports.default = (function (pagination, sort) {
    var cursor = pagination.cursor;
    if (cursor === Cursor_1.start) {
        return {};
    }
    var cursorObj = JSON.parse(atob(cursor));
    var sortKeys = Object.keys(sort);
    var sortKeyFilters = sortKeys.map(function (sortKey, keyIndex) {
        var sortKeysToMatch = sortKeys.slice(0, keyIndex);
        var matchFilter = sortKeysToMatch.reduce(function (result, sortKeyToMatch) {
            result[sortKeyToMatch] = cursorObj[sortKeyToMatch];
            return result;
        }, {});
        var cursorKeyFilter = getCursorKeyFilter(sortKey, sort, pagination, cursorObj);
        matchFilter[sortKey] = cursorKeyFilter;
        return matchFilter;
    });
    var filter = { $or: sortKeyFilters };
    return filter;
});
//# sourceMappingURL=index.js.map